###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.18 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS
import OFS.Folder
import Products
import base_item
import mail_gateway
import mailbox
import thread
import cStringIO
import re
import message_container
import news_group_folder
import message_filter_folder
import imap4_wrapper
import pop3_wrapper
import smtplib
import string
import message_container_sort
import rfc822_address_history
import DateTime

from Products import RFC822Message

gum_add_redirect = HTMLFile('gum_add_redirect', globals())
manage_add_gum_form = HTMLFile('gum_add', globals())


def manage_add_gum(self, id, local_username, email_address, title,
				local_password='local_password',
				local_protocol='pop3',
				mailhost='localhost',
				smtphost='localhost',
				REQUEST=None, RESPONSE=None):
    "Add an instance of the gum class"

    self._setObject(id, gum(id, title, local_username, email_address,
				mailhost, smtphost,
				local_password,
				local_protocol))

    if REQUEST is not None:

		RESPONSE.redirect(self.absolute_url() + '/' + id)


class gum(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	_lock = None
	counter = 0

	message_container_types = message_container.message_container.types

	meta_type = 'GUM'
	icon = 'misc_/GUM/Folder_icon.gif'
	#default_catalog = 'Catalog'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'local_username', 'type':'string'},
		{'id':'local_password', 'type':'string'},
		{'id':'local_protocol', 'type':'string'},
		{'id':'mailhost', 'type':'string'},
		{'id':'smtphost', 'type':'string'},
		{'id':'email_address', 'type':'string'},
		{'id':'default_newshost', 'type':'string'},
		{'id':'listing_size', 'type':'int'},
		{'id':'signature', 'type':'text'},
		{'id':'out_of_office', 'type':'boolean', 'boolean_widget':'out_of_office_choices'},
		{'id':'out_of_office_message', 'type':'text'},
		)

	out_of_office_choices = \
			{
				0:'No',
				1:'Yes'
			}

	meta_types = (
		{'name':'MailGateway',
		 'action':'manage_add_mail_gateway_form'},
		{'name':'MessageContainer',
		 'action':'manage_add_message_container_form'},
		{'name':'NewsGroupFolder',
		 'action':'manage_add_news_group_folder_form'},
		#{'name':'MessageFilterFolder',
		# 'action':'manage_add_message_filter_folder_form'}
		)

	manage_add_message_container_sort = message_container_sort.manage_add_message_container_sort

	manage_add_mail_gateway_form = mail_gateway.manage_add_mail_gateway_form
	manage_add_mail_gateway = mail_gateway.manage_add_mail_gateway
	mail_gateway_add_redirect = mail_gateway.add_mail_gateway_redirect

	manage_add_message_container_form = message_container.manage_add_message_container_form
	manage_add_message_container = message_container.manage_add_message_container
	message_container_add_redirect = message_container.message_container_add_redirect

	manage_add_news_group_folder_form = news_group_folder.manage_add_news_group_folder_form
	manage_add_news_group_folder = news_group_folder.manage_add_news_group_folder
	news_group_folder_add_redirect = news_group_folder.news_group_folder_add_redirect

	manage_add_message_filter_folder_form = message_filter_folder.manage_add_message_filter_folder_form
	manage_add_message_filter_folder = message_filter_folder.manage_add_message_filter_folder
	message_filter_folder_add_redirect = message_filter_folder.message_filter_folder_add_redirect	

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add MailGateway',
		 ('manage_add_mail_gateway', 'manage_add_mail_gateway_form')),
		('Add MessageContainer',
		 ('manage_add_message_container', 'manage_add_message_container_form')),
		('View GUMs',
		 ('',)),
		)

	index_html = HTMLFile('gum_index', globals())

	#base_edit = HTMLFile('gum_edit', globals())

	preferences = HTMLFile('gum_preferences', globals())

	display = HTMLFile('gum_display', globals())

	message_container_global_view = HTMLFile('gum_message_container_global_view', globals())
	message_container_global_view_unread = HTMLFile('gum_message_container_global_view_unread', globals())
	message_filter_priorities = HTMLFile('gum_message_filter_priorities', globals())

	product_name = 'GUM'


	def __init__(self, id,
			title,
			local_username,
			email_address,
			mailhost,
			smtphost,
			local_password,
			local_protocol,
			signature=''):
		"Initialize the class instance"

		self.primary_incoming_folder = 'Inbox'
		self.primary_outgoing_folder = 'Outgoing'
		self.primary_draft_folder = 'Drafts'
		self.primary_template_folder = 'Templates'

		message_container.manage_add_message_container(self, 'Inbox', 'Incoming messages', type=[0])
		message_container.manage_add_message_container(self, 'Trash', 'Deleted messages', type=[0], id='Trash')
		message_container.manage_add_message_container(self, 'Sent items',
								'Outgoing messages', type=[1])		
		message_container.manage_add_message_container(self, 'Drafts', 'Templates that save time', type=[2])
		message_container.manage_add_message_container(self, 'Templates',
								'Template', type=[3])

		rfc822_address_history.manage_add_rfc822_address_history(self, 'IncomingHistory', 'Your incoming address history')
		rfc822_address_history.manage_add_rfc822_address_history(self, 'OutgoingHistory', 'Your outgoing address history')

		self.id = id
		self.title = title

		self.local_username = local_username
		self.local_password = local_password
		self.local_protocol = local_protocol
		self.mailhost = mailhost
		self.smtphost = smtphost
		self.email_address = email_address
		self.signature = signature
		self.out_of_office = 0
		self.out_of_office_message = ''
		self.default_newshost = ''
		self.trash_folder = 'Trash'

		self.listing_size = 15

		try:
		
			self.index_object()

		except AttributeError:

			# No ZCatalog instance available.

			pass

		self.counter = 10
		self.priority_count = 0

		self.notifications = []


	def get_preferences_url(self):
		"""Returns the preferences URL."""

		return self.relative_url() + '/preferences'


	def get_new_priority(self):
		"""Returns a new priority."""

		self.priority_count = self.priority_count + 1

		return self.priority_count


	def render_message_container_type(self, message_container_type):
		"""Renders the name of the message container type."""

		return self.message_container_types[int(message_container_type)]


	def render_message_container_title(self, message_container_id):
		"""Renders the name of the message container."""

		return self[message_container_id].title


	def get_unique_id(self):
		"""Returns a unique identifier."""

		"""
		while 1:

			if self._lock:

				time.sleep(0.1)
				continue

			else:

				self._lock = 1
				self.counter = self.counter + 1
				new_id = str(self.counter)
				self._lock = 0

		return new_id
		"""

		try:

			return self.getParentNode().get_unique_id()

		except AttributeError:

			self.counter = self.counter + 1

			return str(self.counter)


	# FIXME
	# We need to add a primary folder selection
	# tool in the preferences menu


	def get_trash_folder(self):
		"""Returns the trash (deleted items) folder."""

		return self[self.trash_folder]


	def get_primary_copy_to_folder(self):
		"""Returns the primary copy to folder."""

		return self.get_message_containers(type=1)[0]


	def get_primary_incoming_folder(self):
		"""Returns the primary incoming folder."""

		try:

			return self[self.primary_incoming_folder]

		except KeyError:

			return self.get_message_containers(type=0)[0]


	def get_primary_outgoing_folder(self):
		"""Returns the primary outgoing folder."""

		try:

			return self[self.primary_outgoing_folder]

		except KeyError:

			return self.get_message_containers(type=1)[0]


	def get_primary_draft_folder(self):
		"""Returns the primary draft folder."""

		try:

			return self[self.primary_draft_folder]

		except KeyError:

			return self.get_message_containers(type=2)[0]


	def get_primary_template_folder(self):
		"""Returns the template folder."""

		try:

			return self[self.primary_template_folder]

		except KeyError:

			return self.get_message_containers(type=3)[0]


	def get_gum(self):
		"""Returns the GUM instance."""

		return self


	def get_mail_gateway(self):
		"""Returns default mail gateway."""

		try:

			return self[self.default_mail_gateway]

		except AttributeError:

			return self.objectValues('MailGateway')[0]


	def get_news_host(self):
		"""Returns a newshost."""

		# FIXME -- this is not (does not feel like) the right
		# way to do it..

		news_hosts = []

		reference = self

		while 1:

			try:

				news_hosts = reference.objectValues('NewsHost')

			# We've come to the end of the line

			except AttributeError:

				break

			if news_hosts:

				break

			reference = reference.getParentNode()

		if not news_hosts:

			raise 'NoNewsHosts', 'No news hosts are configured on this server'

		return news_hosts[0]


	def get_gum_version(self):
		"""Returns a version number."""

		return open(SOFTWARE_HOME + '/Products/' + self.product_name \
					+ '/version.txt').read()


	def get_gum_website(self):
		"""Returns the GUM website."""

		return open(SOFTWARE_HOME + '/Products/' + self.product_name \
				+ '/website.txt').read()


	def get_address_book(self):
		"""Returns an AddressBook instance if available."""

		object =  self.get_nearest_object_by_meta_type('AddressBook')

		return object


	def manage_out_of_office_post(self):
		"""Updates the out of office settings."""

		pass


	def get_number_of_mail_messages(self):
		"""Gets the number of unread messages in all incoming mailboxes."""

		count = 0

		for message_container in self.get_message_containers(type=0):

			count = count + message_container.get_number_of_messages()

		return count


	def get_number_of_unread_mail_messages(self):
		"""Gets the number of unread messages in all mailboxes."""

		count = 0

		for message_container in self.objectValues('MessageContainer'):

			count = count + message_container.get_number_of_unread_messages()

		return count


	def get_message_filters(self):
		"""Returns all message filters sorted by priority."""

		message_filters = []

		for message_container in self.get_message_containers(type=0):

			message_filters.extend(message_container.objectValues('MessageFilter'))

		message_filters.sort(lambda x,y: cmp(x.priority, y.priority))

		return message_filters

		#FIXME

		message_filters = map(lambda x: x[-1],
					self.ZopeFind(self, obj_metatypes=['MessageFilter'], search_sub=1))

		message_filters.sort(lambda x,y: cmp(x.priority, y.priority))

		#return message_filters


	def render_message_container_type_widget(self, REQUEST):
		"""Renders an HTML widget."""

		select_widget = """<select name="message_container_type:int">"""

		for option in self.message_container_types.keys():

			if REQUEST['message_container_type'] == option:

				selected = " selected"

			else:

				selected = ""

			select_widget = select_widget + """<option value="%s"%s>%s</option>""" % \
						(option, selected, self.message_container_types[option])

		return select_widget + "</select>"


	def list_message_containers(self, all=None):
		"""Returns list of message containers."""

		message_containers = []

		if not all:

			message_container_types = [0,3]

		else:

			message_container_types = self.message_container_types.keys()
		message_container_types.sort()

		for container_type in message_container_types:

			message_containers.append({'group_title':self.message_container_types[container_type],
							'group_global_view_url':self.absolute_url() + \
								'/message_container_global_view?' + \
								'message_container_type=%s' % container_type,
							'containers':self.get_message_containers(type=container_type)})

		return message_containers


	def retrieve_messages(self, REQUEST=None, RESPONSE=None,
				run_as_thread=0):
		"""Wrapper so we can choose whether to retrieve messages
		in threaded mode."""

		#if run_as_thread:

		#	thread.start_new_thread(self._retrieve_messages,
		#		(self,),
		#		{'REQUEST':REQUEST, 'RESPONSE':RESPONSE})

		#else:

		self._retrieve_messages(self, REQUEST=REQUEST,
					RESPONSE=RESPONSE)

		return 'no new messages %s' % DateTime.DateTime()

		#RESPONSE.redirect(self.absolute_url() + '/')
	

	def _retrieve_messages(self, hack, REQUEST=None, RESPONSE=None):
		"""Retrieves messages from the local server."""

		message_filters = self.get_message_filters()

		print "message filters: %s" % message_filters

		primary_folder = self.get_primary_incoming_folder()

		if self.local_username and self.local_password and self.local_protocol:

			if self.local_protocol == 'imap':

				con = imap4_wrapper.imap4_wrapper(self.mailhost)

			elif self.local_protocol == 'pop3':

				con = pop3_wrapper.pop3_wrapper(self.mailhost)

			else:

				raise 'UnknownProtocolException', self.local_protocol

			con.login(self.local_username, self.local_password)

			mbox = con.get_all_messages(RESPONSE=RESPONSE)

		else:

			# FIXME -- poor hack
			# Move to updating of properties?

			checkname = re.compile(r'^([A-Z]|[a-z]|[0-9])*')

			if not checkname.search(self.local_username):

				raise 'NoWay', 'You\'re not getting away with this username.. %s'\
						% self.local_username

			mbox = mailbox.UnixMailbox(
				open('/var/spool/mail/'+self.local_username))

		message_count = 1

		while 1:

			self.really_commit_transaction()

			"""
			if not message_count % 5:

				self.really_commit_transaction()

				if RESPONSE is not None:

					RESPONSE.write('5')

				if not message_count % 100:

					RESPONSE.write('\n')
			"""

			_message = mbox.next()

			if not _message:

				break

			_message.fp.seek(0)
			_message = RFC822Message.util.my_string_io(_message.fp.read())

			message = RFC822Message.rfc822_message_parse.rfc822_message_parse(
				_message)

			self.drop_message(message, message_filters=message_filters)

			message_count = message_count + 1


	def gum_enviroment(self):
		"""Returns a truth value to whether the requesting object is within a GUM enviroment."""

		# FIXME -- this is stupid

		return 1


	def compose_mail(self, subject='', body='', recipients=[],
				newsgroups=[], REQUEST=None, RESPONSE=None):
		"""Creates a new instance of a message and redirects to it."""

		subject = subject or ''
		body = body or ''
		recipients = recipients or []
		newsgroups = newsgroups or []

		if string.strip(self.signature):

			signature_append = '\n--\n' + self.signature

		else:

			signature_append = ''


		if REQUEST:

			if REQUEST.has_key('subject') and REQUEST['subject']:

				subject = subject

			if REQUEST.has_key('body') and REQUEST['body']:

				body = body + signature_append

			else:

				body = body + signature_append

			if REQUEST.has_key('recipients') and REQUEST['recipients']:

				recipients = REQUEST['recipients']

				if type(recipients) == type(''):

					recipients = [recipients]

			if REQUEST.has_key('newsgroups') and REQUEST['newsgroups']:

				newsgroups = REQUEST['newsgroups']

				if type(newsgroups) == type(''):

					newsgroups = [newsgroups]


		draft_folder = self.get_primary_draft_folder()

		new_id = str(self.get_unique_id())

		RFC822Message.rfc822_message.manage_add_rfc822_message(draft_folder,
								new_id,
								sender=self.email_address,
								body=body,
								subject=subject)

		message_reference = draft_folder[new_id]
		message_reference.headers['copy-to-folder'] = self.get_primary_copy_to_folder().relative_url()
		message_reference.headers = message_reference.headers

		if recipients:

			for recipient in recipients:

				new_id = str(self.get_unique_id())

				message_reference['to'].manage_add_rfc822_address(
									new_id,
									recipient)

		if newsgroups:

			for newsgroup in newsgroups:

				message_reference.add_newsgroup(newsgroup)


		if RESPONSE:

			RESPONSE.redirect(message_reference.absolute_url())

		else:

			return message_reference


	def get_message_containers(self, type=None):
		"""Returns a list of all available message containers."""

		message_containers = map(lambda x: x[-1],
			self.ZopeFind(self, obj_metatypes=['MessageContainer']))

		if type is not None:

			filter_function = "lambda x: x.type == %s" % type
	
			truth_filter = eval(filter_function)
	
			message_containers = filter(truth_filter, message_containers)

		return message_containers


	def get_newsgroup_folders(self):
		"""Returns a list of all the message containers."""

		return self.objectValues('NewsGroupFolder')


	def manage_preferences(self, REQUEST, RESPONSE):
		"""Manages preferences."""

		self.base_edit_properties(REQUEST)

		RESPONSE.redirect(self.relative_url() + '/preferences')


	def get_all_messages_in_container_type(self, type):
		"""Returns all messages in given container type."""

		message_containers = []
		messages = []

		for message_container in self.get_message_containers(type=type):

			for message in message_container.get_messages():

				messages.append(message)

		return messages


	def get_all_available_newshosts(self):
		"""Returns a list of all available news_host objects."""

		news_hosts = []

		reference = self

		while 1:

			try:

				_news_hosts = reference.objectValues('NewsHost')

			except AttributeError:

				# We've come to the end of the road..

				break

			if _news_hosts:

				news_hosts.extend(_news_hosts)

			reference = reference.getParentNode()

		return news_hosts


	def move_message_filter_up(self, REQUEST, RESPONSE):
		"""Increases the priority of given message filter."""

		priority = int(REQUEST['priority'])
		message_filters = self.get_message_filters()

		for message_filter_index in range(len(message_filters)):

			message_filter = message_filters[message_filter_index]

			if message_filter.priority == priority:

				downgraded_filter = message_filters[message_filter_index - 1]

				new_priority = downgraded_filter.priority
				message_filter.priority = new_priority
				downgraded_filter.priority = priority

				break

		RESPONSE.redirect(self.absolute_url() + '/message_filter_priorities')


	def move_message_filter_down(self, REQUEST, RESPONSE):
		"""Decreases the priority of given message filter."""

		priority = int(REQUEST['priority'])
		message_filters = self.get_message_filters()

		for message_filter_index in range(len(message_filters)):

			message_filter = message_filters[message_filter_index]

			if message_filter.priority == priority:

				try:

					upgraded_filter = message_filters[message_filter_index + 1]

				except IndexError:

					upgraded_filter = message_filters[0]

				new_priority = upgraded_filter.priority
				message_filter.priority = new_priority
				upgraded_filter.priority = priority

				break


		RESPONSE.redirect(self.absolute_url() + '/message_filter_priorities')


	def get_outgoing_address_history(self):
		"""Returns the outgoing address history."""

		return self.OutgoingHistory


	def get_incoming_address_history(self):
		"""Returns the incoming address history."""

		return self.IncomingHistory
	

	def drop_outgoing_address(self, email, realname='', comment=''):
		"""Drops an address into the outgoing address history object."""

		new_id = self.get_unique_id()

		history = self.get_outgoing_address_history()
		# FIXME -- what's ut with TypeErrors on the comment keyword?
		history.manage_add_rfc822_address(new_id, email)
		

	def drop_incoming_address(self, email, realname='', comment=''):
		"""Drops an address into the incmoing address history object."""

		new_id = self.get_unique_id()

		history = self.get_incoming_address_history()
		# FIXME -- what's up with TypeErrors on the comment keyword?
		history.manage_add_rfc822_address(new_id, email)


	def drop_message(self, message, message_filters=None):
		"""Drops a message to a message container with a matching mail filter,
		or alternatively to the default incoming folder."""

		try:

			for recipient in [message.sender] + message.to + message.cc:

				if not recipient[-1] == self.email_address:

					self.drop_incoming_address(email=recipient[-1], realname=recipient[0])

		except Exception, value:

			import sys, traceback, string
			type, val, tb = sys.exc_info()
			sys.stderr.write(string.join(traceback.format_exception(type, val, tb), ''))
			del type, val, tb

		print 'in drop message'
		print 'message filters: %s' % message_filters

		message_filters = message_filters or self.get_message_filters()

		print 'got message filters'

		try:

			if self.out_of_office:

				message_reference = self.compose_mail(subject='Currently out of office',body=self.out_of_office_message,
									recipients=[message.sender[-1]])
				print 'got message reference'
				message_reference.headers['copy-to-folder'] = ''
				print 'reset copy-to-folder'
				message_reference.headers = message_reference.headers
				print 'notified ZODB of changes'

				message_reference.send()
				print 'message sent'

		except Exception, value:

			import sys, traceback, string
			type, val, tb = sys.exc_info()
			sys.stderr.write(string.join(traceback.format_exception(type, val, tb), ''))
			del type, val, tb

		for message_filter in message_filters:

			print "message_filter: %s" % message_filter.relative_url()
			print

			if message_filter.match(message.headers):

				RFC822Message.util.store_message(
					message_filter.getParentNode(),
					message,
					readonly=1)

				break

		else:

			RFC822Message.util.store_message(
				self.get_primary_incoming_folder(),
				message, readonly=1)


		try:

			subject = message.headers['subject']

		except Exception, value:

			subject = '[No subject]'

		self.add_notify_message('A new message has arrived from %s with subject %s' % (message.sender[0], subject))

		print 'dropped a message'


	def add_notify_message(self, message):
		"""Sets the notify status to true."""

		try:

			self.getParentNode().add_notify_message(message)
			print 'used getParentNode to add notify message'

		except AttributeError:

			print 'used our own notifications list'
			self.notifications.append(message)


	def get_notified_messages(self):
		"""Returns a list of notify messages."""

		return self.notifications


	def reset_notified_messages(self):
		"""Resets the notify list."""

		self.notifications = []


	def build_smtp(self):
		"""Returns an SMTP connection to the default SMTP host."""

		return smtplib.SMTP(self.smtphost)


	def post(self, message, raw_message=None):
		"""Deconstructs an rfc822_message object and posts it."""

		smtp_connection = self.build_smtp()

		if raw_message is None:

			raw_message = message.raw_message()

		smtp_connection.sendmail(message.sender, message.get_recipients(),
						raw_message)

		smtp_connection.quit()


default__class_init__(gum)