###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.8 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import base_item
import message_container


def manage_add_message_container_sort(self, title, type=[0], REQUEST=None, RESPONSE=None):
	"Add an instance of the message_container_sort class"

	for type in type:

		new_id = self.get_unique_id()

		self._setObject(new_id, message_container(new_id, title, int(type)))

	if RESPONSE is not None:

		RESPONSE.redirect(self.absolute_url())


class message_container(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'MessageContainerSort'
	icon = 'misc_/Transform/Folder_icon.gif'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'description', 'type':'text'},
		{'id':'type', 'type':'int', 'select_widget':'types'}
		)
		

	types = {
			0:'Incoming',
			1:'Outgoing',
			2:'Draft',
			3:'Template'
		} 

	_properties = (
		{'id':'title', 'type':'string'},
		)

	meta_types = (
			{'name':'MessageContainer', 'action':'manage_add_message_container_form'},
			)

	manage_add_message_container_form = message_container.manage_add_message_container_form
	manage_add_message_container = message_container.manage_add_message_container

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		#('Add Message',
		# ('manage_add_message', 'manage_add_message_orm')),
		('View MessageContainers',
		 ('',)),
		)

	index_html = HTMLFile('message_container_index', globals())


	def __init__(self, id, title='', description='', type=0):
		"Initialize the class instance"

		self.id = id
		self.title = title
		self.type = type

		try:

			self.index_object()

		except AttributeError:

			# No ZCatalog instance available..

			pass


	def get_number_of_messages(self):
		"""Returns the number of messages this object contains."""

		return len(self.get_messages())


	def get_number_of_unread_messages(self):
		"""Returns the number of messages unread that this object
		contains."""

		return len(filter(lambda x: not x.read, self.get_messages()))


	def get_messages(self):
		"""Returns a list of messages contained in this message container."""

		return self.ZopeFind(self, obj_metatypes=['RFC822Message'])


default__class_init__(message_container)

